@@ -65,7 +65,7 @@ require_once($CFG->dirroot . '/mod/lti/tests/mod_lti_testcase.php');
  * @copyright  Copyright (c) 2012 Moodlerooms Inc. (http://www.moodlerooms.com)
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
-final class locallib_test extends mod_lti_testcase {
+class locallib_test extends mod_lti_testcase {
 
     /**
      * @covers ::lti_split_parameters()
@@ -302,21 +302,21 @@ final class locallib_test extends mod_lti_testcase {
 
         // Try when the forcessl config property is not set.
         lti_prepare_type_for_save($type, $config);
-        $this->assertObjectHasAttribute('lti_forcessl', $config);
+        $this->assertObjectHasProperty('lti_forcessl', $config);
         $this->assertEquals(0, $config->lti_forcessl);
         $this->assertEquals(0, $type->forcessl);
 
         // Try when forcessl config property is set.
         $config->lti_forcessl = 1;
         lti_prepare_type_for_save($type, $config);
-        $this->assertObjectHasAttribute('lti_forcessl', $config);
+        $this->assertObjectHasProperty('lti_forcessl', $config);
         $this->assertEquals(1, $config->lti_forcessl);
         $this->assertEquals(1, $type->forcessl);
 
         // Try when forcessl config property is set to 0.
         $config->lti_forcessl = 0;
         lti_prepare_type_for_save($type, $config);
-        $this->assertObjectHasAttribute('lti_forcessl', $config);
+        $this->assertObjectHasProperty('lti_forcessl', $config);
         $this->assertEquals(0, $config->lti_forcessl);
         $this->assertEquals(0, $type->forcessl);
     }
@@ -517,7 +517,7 @@ final class locallib_test extends mod_lti_testcase {
      *
      * @return array of [urlToTest, expectedTool, allTools]
      */
-    public static function lti_get_best_tool_by_url_provider(): array {
+    public function lti_get_best_tool_by_url_provider() {
         $tools = [
             (object) [
                 'name' => 'Here',
@@ -605,7 +605,7 @@ final class locallib_test extends mod_lti_testcase {
         // Construct the final array as required by the provider API. Each row
         // of the array contains the URL to test, the expected tool, and
         // the complete list of tools.
-        return array_map(function($data) use ($tools): array {
+        return array_map(function($data) use ($tools) {
             return [$data['url'], $data['expected'], $tools];
         }, $data);
     }
@@ -623,6 +623,111 @@ final class locallib_test extends mod_lti_testcase {
         $this->assertSame($expected, $actual);
     }
 
+    /**
+     * @covers ::lti_get_tools_by_domain()
+     *
+     * Test lti_get_tools_by_domain.
+     */
+    public function test_lti_get_tools_by_domain() {
+        $this->resetAfterTest();
+
+        /** @var \mod_lti_generator $ltigenerator */
+        $ltigenerator = $this->getDataGenerator()->get_plugin_generator('mod_lti');
+
+        // Create a tool type with good domain.
+        $ltigenerator->create_tool_types([
+            'name' => 'Test tool 1',
+            'description' => 'Good example description',
+            'tooldomain' => 'example.com',
+            'baseurl' => 'https://example.com/i/am/?where=here',
+            'state' => LTI_TOOL_STATE_CONFIGURED
+        ]);
+
+        // Create a tool type with bad domain.
+        $ltigenerator->create_tool_types([
+            'name' => 'Test tool 2',
+            'description' => 'Bad example description',
+            'tooldomain' => 'badexample.com',
+            'baseurl' => 'https://badexample.com/i/am/?where=here',
+            'state' => LTI_TOOL_STATE_CONFIGURED
+        ]);
+
+        $records = lti_get_tools_by_domain('example.com', LTI_TOOL_STATE_CONFIGURED);
+        $this->assertCount(1, $records);
+        $this->assertEmpty(array_diff(
+            ['https://example.com/i/am/?where=here'],
+            array_column($records, 'baseurl')
+        ));
+    }
+
+    /**
+     * @covers ::lti_get_tools_by_domain()
+     *
+     * Test test_lti_get_tools_by_domain_restrict_types_category.
+     */
+    public function test_lti_get_tools_by_domain_restrict_types_category() {
+        $this->resetAfterTest();
+
+        $coursecat1 = $this->getDataGenerator()->create_category();
+        $coursecat2 = $this->getDataGenerator()->create_category();
+
+        $course1 = $this->getDataGenerator()->create_course(['category' => $coursecat1->id]);
+        $course2 = $this->getDataGenerator()->create_course(['category' => $coursecat2->id]);
+
+        /** @var \mod_lti_generator $ltigenerator */
+        $ltigenerator = $this->getDataGenerator()->get_plugin_generator('mod_lti');
+
+        // Create a tool type with domain restricting to a category1.
+        $ltigenerator->create_tool_types([
+            'name' => 'Test tool 1',
+            'description' => 'Good example description',
+            'tooldomain' => 'exampleone.com',
+            'baseurl' => 'https://exampleone.com/tool/1',
+            'state' => LTI_TOOL_STATE_CONFIGURED,
+            'lti_coursecategories' => $coursecat1->id
+        ]);
+
+        // Create another tool type using the same domain, restricted to category2.
+        $ltigenerator->create_tool_types([
+            'name' => 'Test tool 1',
+            'description' => 'Good example description',
+            'tooldomain' => 'exampleone.com',
+            'baseurl' => 'https://exampleone.com/tool/2',
+            'state' => LTI_TOOL_STATE_CONFIGURED,
+            'lti_coursecategories' => $coursecat2->id
+        ]);
+
+        // Create a tool type with domain restricting to a category2.
+        $ltigenerator->create_tool_types([
+            'name' => 'Test tool 2',
+            'description' => 'Good example description',
+            'tooldomain' => 'exampletwo.com',
+            'baseurl' => 'https://exampletwo.com/tool/3',
+            'state' => LTI_TOOL_STATE_CONFIGURED,
+            'lti_coursecategories' => $coursecat2->id
+        ]);
+
+        // Get tool types for domain 'exampleone' in course 1 and verify only the one result under course category 1 is included.
+        $records = lti_get_tools_by_domain('exampleone.com', LTI_TOOL_STATE_CONFIGURED, $course1->id);
+        $this->assertCount(1, $records);
+        $this->assertEmpty(array_diff(
+            ['https://exampleone.com/tool/1'],
+            array_column($records, 'baseurl')
+        ));
+
+        // Get tool types for domain 'exampleone' in course 2 and verify only the one result under course category 2 is included.
+        $records = lti_get_tools_by_domain('exampleone.com', LTI_TOOL_STATE_CONFIGURED, $course2->id);
+        $this->assertCount(1, $records);
+        $this->assertEmpty(array_diff(
+            ['https://exampleone.com/tool/2'],
+            array_column($records, 'baseurl')
+        ));
+
+        // Get tool types for domain 'exampletwo' in course 1 and verify that no results are found.
+        $records = lti_get_tools_by_domain('exampletwo.com', LTI_TOOL_STATE_CONFIGURED, $course1->id);
+        $this->assertCount(0, $records);
+    }
+
     /**
      * Test lti_get_jwt_message_type_mapping().
      */
@@ -1915,7 +2020,7 @@ MwIDAQAB
      *
      * @return array[] the test case data.
      */
-    public static function lti_get_ims_role_provider(): array {
+    public function lti_get_ims_role_provider() {
         return [
             'Student, LTI 1.1, no role switch' => [
                 'islti2' => false,
@@ -2159,4 +2264,118 @@ MwIDAQAB
 
         return ['proxies' => $proxies, 'types' => $types];
     }
+
+    /**
+     * Test for lti_get_lti_types_by_course.
+     *
+     * Note: This includes verification of the broken legacy behaviour in which the inclusion of course and site tools could be
+     * controlled independently, based on the capabilities 'mod/lti:addmanualinstance' (to include course tools) and
+     * 'mod/lti:addpreconfiguredinstance' (to include site tools). This behaviour is deprecated in 4.3 and all preconfigured tools
+     * are controlled by the single capability 'mod/lti:addpreconfiguredinstance'.
+     *
+     * @covers ::lti_get_lti_types_by_course()
+     * @return void
+     */
+    public function test_lti_get_lti_types_by_course(): void {
+        $this->resetAfterTest();
+
+        global $DB;
+        $coursecat1 = $this->getDataGenerator()->create_category();
+        $coursecat2 = $this->getDataGenerator()->create_category();
+        $course = $this->getDataGenerator()->create_course(['category' => $coursecat1->id]);
+        $course2 = $this->getDataGenerator()->create_course(['category' => $coursecat2->id]);
+        $teacher = $this->getDataGenerator()->create_and_enrol($course, 'editingteacher');
+        $teacher2 = $this->getDataGenerator()->create_and_enrol($course2, 'editingteacher');
+
+        // Create the following tool types for testing:
+        // - Site tool configured as "Do not show" (LTI_COURSEVISIBLE_NO).
+        // - Site tool configured as "Show as a preconfigured tool only" (LTI_COURSEVISIBLE_PRECONFIGURED).
+        // - Site tool configured as "Show as a preconfigured tool and in the activity chooser" (LTI_COURSEVISIBLE_ACTIVITYCHOOSER).
+        // - Course tool which, by default, is configured as LTI_COURSEVISIBLE_ACTIVITYCHOOSER).
+        // - Site tool configured to "Show as a preconfigured tool and in the activity chooser" but restricted to a category.
+
+        /** @var \mod_lti_generator $ltigenerator */
+        $ltigenerator = $this->getDataGenerator()->get_plugin_generator('mod_lti');
+        $ltigenerator->create_tool_types([
+            'name' => 'site tool do not show',
+            'baseurl' => 'http://example.com/tool/1',
+            'coursevisible' => LTI_COURSEVISIBLE_NO,
+            'state' => LTI_TOOL_STATE_CONFIGURED
+        ]);
+        $ltigenerator->create_tool_types([
+            'name' => 'site tool preconfigured only',
+            'baseurl' => 'http://example.com/tool/2',
+            'coursevisible' => LTI_COURSEVISIBLE_PRECONFIGURED,
+            'state' => LTI_TOOL_STATE_CONFIGURED
+        ]);
+        $ltigenerator->create_tool_types([
+            'name' => 'site tool preconfigured and activity chooser',
+            'baseurl' => 'http://example.com/tool/3',
+            'coursevisible' => LTI_COURSEVISIBLE_ACTIVITYCHOOSER,
+            'state' => LTI_TOOL_STATE_CONFIGURED
+        ]);
+        $ltigenerator->create_course_tool_types([
+            'name' => 'course tool preconfigured and activity chooser',
+            'baseurl' => 'http://example.com/tool/4',
+            'course' => $course->id
+        ]);
+        $ltigenerator->create_tool_types([
+            'name' => 'site tool preconfigured and activity chooser, restricted to category 2',
+            'baseurl' => 'http://example.com/tool/5',
+            'coursevisible' => LTI_COURSEVISIBLE_ACTIVITYCHOOSER,
+            'state' => LTI_TOOL_STATE_CONFIGURED,
+            'lti_coursecategories' => $coursecat2->id
+        ]);
+
+        $this->setUser($teacher); // Important: this deprecated method depends on the global user for cap checks.
+
+        // Request using the default 'coursevisible' param will include all tools except the one configured as "Do not show".
+        $coursetooltypes = lti_get_lti_types_by_course($course->id);
+        $this->assertDebuggingCalled();
+        $this->assertCount(3, $coursetooltypes);
+        $this->assertEmpty(array_diff(
+            ['http://example.com/tool/2', 'http://example.com/tool/3', 'http://example.com/tool/4'],
+            array_column($coursetooltypes, 'baseurl')
+        ));
+
+        // Request for only those tools configured to show in the activity chooser for the teacher.
+        $coursetooltypes = lti_get_lti_types_by_course($course->id, [LTI_COURSEVISIBLE_ACTIVITYCHOOSER]);
+        $this->assertDebuggingCalled();
+        $this->assertCount(2, $coursetooltypes);
+        $this->assertEmpty(array_diff(
+            ['http://example.com/tool/3', 'http://example.com/tool/4'],
+            array_column($coursetooltypes, 'baseurl')
+        ));
+
+        // Request for only those tools configured to show as a preconfigured tool for the teacher.
+        $coursetooltypes = lti_get_lti_types_by_course($course->id, [LTI_COURSEVISIBLE_PRECONFIGURED]);
+        $this->assertDebuggingCalled();
+        $this->assertCount(1, $coursetooltypes);
+        $this->assertEmpty(array_diff(
+            ['http://example.com/tool/2'],
+            array_column($coursetooltypes, 'baseurl')
+        ));
+
+        // Request for teacher2 in course2 (course category 2).
+        $this->setUser($teacher2);
+        $coursetooltypes = lti_get_lti_types_by_course($course2->id);
+        $this->assertDebuggingCalled();
+        $this->assertCount(3, $coursetooltypes);
+        $this->assertEmpty(array_diff(
+            ['http://example.com/tool/2', 'http://example.com/tool/3', 'http://example.com/tool/5'],
+            array_column($coursetooltypes, 'baseurl')
+        ));
+
+        // Request for a teacher who cannot use preconfigured tools in the course.
+        // No tools are available.
+        $this->setUser($teacher);
+        $teacherrole = $DB->get_record('role', array('shortname' => 'editingteacher'));
+        assign_capability('mod/lti:addpreconfiguredinstance', CAP_PROHIBIT, $teacherrole->id,
+            \core\context\course::instance($course->id));
+        $coursetooltypes = lti_get_lti_types_by_course($course->id);
+        $this->assertDebuggingCalled();
+        $this->assertCount(0, $coursetooltypes);
+        $this->unassignUserCapability('mod/lti:addpreconfiguredinstance', (\core\context\course::instance($course->id))->id,
+            $teacherrole->id);
+    }
 }
