@@ -22,7 +22,11 @@
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 
+use core\report_helper;
+
 defined('MOODLE_INTERNAL') || die;
+global $CFG;
+require_once($CFG->libdir . '/tablelib.php');
 
 /**
  * Table log class for displaying logs.
@@ -32,7 +36,6 @@ defined('MOODLE_INTERNAL') || die;
  * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
  */
 class report_log_table_log extends table_sql {
-
     /** @var array list of user fullnames shown in report */
     private $userfullnames = array();
 
@@ -42,6 +45,9 @@ class report_log_table_log extends table_sql {
     /** @var stdClass filters parameters */
     private $filterparams;
 
+    /** @var int[] A list of users to filter by */
+    private ?array $lateuseridfilter = null;
+
     /**
      * Sets up the table_log parameters.
      *
@@ -59,7 +65,7 @@ class report_log_table_log extends table_sql {
     public function __construct($uniqueid, $filterparams = null) {
         parent::__construct($uniqueid);
 
-        $this->set_attribute('class', 'reportlog generaltable generalbox');
+        $this->set_attribute('class', 'reportlog generaltable generalbox table-sm');
         $this->filterparams = $filterparams;
         // Add course column if logs are displayed for site.
         $cols = array();
@@ -108,29 +114,30 @@ class report_log_table_log extends table_sql {
      * @return string|false
      */
     protected function get_user_fullname($userid) {
-        global $DB;
-
         if (empty($userid)) {
             return false;
         }
 
-        if (!empty($this->userfullnames[$userid])) {
-            return $this->userfullnames[$userid];
+        // Check if we already have this users' fullname.
+        $userfullname = $this->userfullnames[$userid] ?? null;
+        if (!empty($userfullname)) {
+            return $userfullname;
         }
 
         // We already looked for the user and it does not exist.
-        if ($this->userfullnames[$userid] === false) {
+        if ($userfullname === false) {
             return false;
         }
 
         // If we reach that point new users logs have been generated since the last users db query.
-        list($usql, $uparams) = $DB->get_in_or_equal($userid);
-        $sql = "SELECT id," . get_all_user_name_fields(true) . " FROM {user} WHERE id " . $usql;
-        if (!$user = $DB->get_records_sql($sql, $uparams)) {
-            return false;
+        $userfieldsapi = \core_user\fields::for_name();
+        $fields = $userfieldsapi->get_sql('', false, '', '', false)->selects;
+        if ($user = \core_user::get_user($userid, $fields)) {
+            $this->userfullnames[$userid] = fullname($user, has_capability('moodle/site:viewfullnames', $this->get_context()));
+        } else {
+            $this->userfullnames[$userid] = false;
         }
 
-        $this->userfullnames[$userid] = fullname($user);
         return $this->userfullnames[$userid];
     }
 
@@ -143,9 +150,9 @@ class report_log_table_log extends table_sql {
     public function col_time($event) {
 
         if (empty($this->download)) {
-            $dateformat = get_string('strftimerecent', 'core_langconfig');
+            $dateformat = get_string('strftimedatetimeaccurate', 'core_langconfig');
         } else {
-            $dateformat = get_string('strftimedatetimeshort', 'core_langconfig');
+            $dateformat = get_string('strftimedatetimeshortaccurate', 'core_langconfig');
         }
         return userdate($event->timecreated, $dateformat);
     }
@@ -153,20 +160,22 @@ class report_log_table_log extends table_sql {
     /**
      * Generate the username column.
      *
-     * @param stdClass $event event data.
+     * @param \core\event\base $event event data.
      * @return string HTML for the username column
      */
     public function col_fullnameuser($event) {
         // Get extra event data for origin and realuserid.
         $logextra = $event->get_logextra();
 
+        $eventusername = $event->userid ? $this->get_user_fullname($event->userid) : false;
+
         // Add username who did the action.
         if (!empty($logextra['realuserid'])) {
             $a = new stdClass();
             if (!$a->realusername = $this->get_user_fullname($logextra['realuserid'])) {
                 $a->realusername = '-';
             }
-            if (!$a->asusername = $this->get_user_fullname($event->userid)) {
+            if (!$a->asusername = $eventusername) {
                 $a->asusername = '-';
             }
             if (empty($this->download)) {
@@ -180,13 +189,15 @@ class report_log_table_log extends table_sql {
             }
             $username = get_string('eventloggedas', 'report_log', $a);
 
-        } else if (!empty($event->userid) && $username = $this->get_user_fullname($event->userid)) {
+        } else if ($eventusername) {
             if (empty($this->download)) {
-                $params = array('id' => $event->userid);
+                $params = ['id' => $event->userid];
                 if ($event->courseid) {
                     $params['course'] = $event->courseid;
                 }
-                $username = html_writer::link(new moodle_url('/user/view.php', $params), $username);
+                $username = html_writer::link(new moodle_url('/user/view.php', $params), $eventusername);
+            } else {
+                $username = $eventusername;
             }
         } else {
             $username = '-';
@@ -273,12 +284,7 @@ class report_log_table_log extends table_sql {
      */
     public function col_eventname($event) {
         // Event name.
-        if ($this->filterparams->logreader instanceof logstore_legacy\log\store) {
-            // Hack for support of logstore_legacy.
-            $eventname = $event->eventname;
-        } else {
-            $eventname = $event->get_name();
-        }
+        $eventname = $event->get_name();
         // Only encode as an action link if we're not downloading.
         if (($url = $event->get_url()) && empty($this->download)) {
             $eventname = $this->action_link($url, $eventname, 'action');
@@ -294,7 +300,7 @@ class report_log_table_log extends table_sql {
      */
     public function col_description($event) {
         // Description.
-        return $event->get_description();
+        return format_text($event->get_description(), FORMAT_PLAIN);
     }
 
     /**
@@ -323,7 +329,7 @@ class report_log_table_log extends table_sql {
         $ip = $logextra['ip'];
 
         if (empty($this->download)) {
-            $url = new moodle_url("/iplookup/index.php?ip={$ip}&user={$event->userid}");
+            $url = new moodle_url("/iplookup/index.php?popup=1&ip={$ip}&user={$event->userid}");
             $ip = $this->action_link($url, $ip, 'ip');
         }
         return $ip;
@@ -369,19 +375,10 @@ class report_log_table_log extends table_sql {
         global $DB;
 
         // In new logs we have a field to pick, and in legacy try get this from action.
-        if ($this->filterparams->logreader instanceof logstore_legacy\log\store) {
-            $action = $this->get_legacy_crud_action($this->filterparams->action);
-            $firstletter = substr($action, 0, 1);
-            if ($firstletter == '-') {
-                $sql = $DB->sql_like('action', ':action', false, true, true);
-                $params['action'] = '%'.substr($action, 1).'%';
-            } else {
-                $sql = $DB->sql_like('action', ':action', false);
-                $params['action'] = '%'.$action.'%';
-            }
-        } else if (!empty($this->filterparams->action)) {
-            $sql = "crud = :crud";
-            $params['crud'] = $this->filterparams->action;
+        if (!empty($this->filterparams->action)) {
+             list($sql, $params) = $DB->get_in_or_equal(str_split($this->filterparams->action),
+                    SQL_PARAMS_NAMED, 'crud');
+            $sql = "crud " . $sql;
         } else {
             // Add condition for all possible values of crud (to use db index).
             list($sql, $params) = $DB->get_in_or_equal(array('c', 'r', 'u', 'd'),
@@ -400,16 +397,10 @@ class report_log_table_log extends table_sql {
         $joins = array();
         $params = array();
 
-        if ($this->filterparams->logreader instanceof logstore_legacy\log\store) {
-            // The legacy store doesn't support context level.
-            $joins[] = "cmid = :cmid";
-            $params['cmid'] = $this->filterparams->modid;
-        } else {
-            $joins[] = "contextinstanceid = :contextinstanceid";
-            $joins[] = "contextlevel = :contextmodule";
-            $params['contextinstanceid'] = $this->filterparams->modid;
-            $params['contextmodule'] = CONTEXT_MODULE;
-        }
+        $joins[] = "contextinstanceid = :contextinstanceid";
+        $joins[] = "contextlevel = :contextmodule";
+        $params['contextinstanceid'] = $this->filterparams->modid;
+        $params['contextmodule'] = CONTEXT_MODULE;
 
         $sql = implode(' AND ', $joins);
         return array($sql, $params);
@@ -422,21 +413,15 @@ class report_log_table_log extends table_sql {
      * @param bool $useinitialsbar do you want to use the initials bar.
      */
     public function query_db($pagesize, $useinitialsbar = true) {
-        global $DB;
+        global $DB, $USER;
 
         $joins = array();
         $params = array();
 
         // If we filter by userid and module id we also need to filter by crud and edulevel to ensure DB index is engaged.
-        $useextendeddbindex = !($this->filterparams->logreader instanceof logstore_legacy\log\store)
-                && !empty($this->filterparams->userid) && !empty($this->filterparams->modid);
+        $useextendeddbindex = !empty($this->filterparams->userid) && !empty($this->filterparams->modid);
 
-        $groupid = 0;
         if (!empty($this->filterparams->courseid) && $this->filterparams->courseid != SITEID) {
-            if (!empty($this->filterparams->groupid)) {
-                $groupid = $this->filterparams->groupid;
-            }
-
             $joins[] = "courseid = :courseid";
             $params['courseid'] = $this->filterparams->courseid;
         }
@@ -458,17 +443,14 @@ class report_log_table_log extends table_sql {
         }
 
         // Getting all members of a group.
-        if ($groupid and empty($this->filterparams->userid)) {
-            if ($gusers = groups_get_members($groupid)) {
-                $gusers = array_keys($gusers);
-                $joins[] = 'userid IN (' . implode(',', $gusers) . ')';
-            } else {
-                $joins[] = 'userid = 0'; // No users in groups, so we want something that will always be false.
-            }
-        } else if (!empty($this->filterparams->userid)) {
-            $joins[] = "userid = :userid";
-            $params['userid'] = $this->filterparams->userid;
-        }
+        [
+            'joins' => $groupjoins,
+            'params' => $groupparams,
+            'useridfilter' => $this->lateuseridfilter,
+        ] = report_helper::get_group_filter($this->filterparams);
+
+        $joins = array_merge($joins, $groupjoins);
+        $params = array_merge($params, $groupparams);
 
         if (!empty($this->filterparams->date)) {
             $joins[] = "timecreated > :date AND timecreated < :enddate";
@@ -486,8 +468,28 @@ class report_log_table_log extends table_sql {
             $params = array_merge($params, $edulevelparams);
         }
 
-        if (!($this->filterparams->logreader instanceof logstore_legacy\log\store)) {
-            // Filter out anonymous actions, this is N/A for legacy log because it never stores them.
+        // Origin.
+        if (isset($this->filterparams->origin) && ($this->filterparams->origin != '')) {
+            if ($this->filterparams->origin !== '---') {
+                // Filter by a single origin.
+                $joins[] = "origin = :origin";
+                $params['origin'] = $this->filterparams->origin;
+            } else {
+                // Filter by everything else.
+                list($originsql, $originparams) = $DB->get_in_or_equal(array('cli', 'restore', 'ws', 'web'),
+                    SQL_PARAMS_NAMED, 'origin', false);
+                $joins[] = "origin " . $originsql;
+                $params = array_merge($params, $originparams);
+            }
+        }
+
+        // Filter out anonymous actions, this is N/A for legacy log because it never stores them.
+        if ($this->filterparams->modid) {
+            $context = context_module::instance($this->filterparams->modid);
+        } else {
+            $context = context_course::instance($this->filterparams->courseid);
+        }
+        if (!has_capability('moodle/site:viewanonymousevents', $context)) {
             $joins[] = "anonymous = 0";
         }
 
@@ -500,24 +502,29 @@ class report_log_table_log extends table_sql {
             $this->pageable(false);
         }
 
-        // Get the users and course data.
-        $this->rawdata = $this->filterparams->logreader->get_events_select_iterator($selector, $params,
-            $this->filterparams->orderby, $this->get_page_start(), $this->get_page_size());
-
-        // Update list of users which will be displayed on log page.
-        $this->update_users_used();
-
         // Get the events. Same query than before; even if it is not likely, logs from new users
         // may be added since last query so we will need to work around later to prevent problems.
         // In almost most of the cases this will be better than having two opened recordsets.
-        $this->rawdata = $this->filterparams->logreader->get_events_select_iterator($selector, $params,
-            $this->filterparams->orderby, $this->get_page_start(), $this->get_page_size());
+        $this->rawdata = new \CallbackFilterIterator(
+            $this->filterparams->logreader->get_events_select_iterator(
+                $selector,
+                $params,
+                $this->filterparams->orderby,
+                $this->get_page_start(),
+                $this->get_page_size(),
+            ),
+            function ($event) {
+                if ($this->lateuseridfilter === null) {
+                    return true;
+                }
+                return isset($this->lateuseridfilter[$event->userid]);
+            },
+        );
 
         // Set initial bars.
         if ($useinitialsbar && !$this->is_downloading()) {
             $this->initialbars($total > $pagesize);
         }
-
     }
 
     /**
@@ -529,55 +536,6 @@ class report_log_table_log extends table_sql {
      * @deprecated since Moodle 2.9 MDL-48595 - please do not use this function any more.
      */
     public function update_users_and_courses_used() {
-        throw new coding_exception('update_users_and_courses_used() can not be used any more, please use update_users_used() instead.');
-    }
-
-    /**
-     * Helper function to create list of user fullnames shown in log report.
-     *
-     * This will update $this->userfullnames array with userfullname,
-     * which will be used to render logs in table.
-     *
-     * @since   Moodle 2.9
-     * @return  void
-     */
-    protected function update_users_used() {
-        global $DB;
-
-        $this->userfullnames = array();
-        $userids = array();
-
-        // For each event cache full username.
-        // Get list of userids which will be shown in log report.
-        foreach ($this->rawdata as $event) {
-            $logextra = $event->get_logextra();
-            if (!empty($event->userid) && empty($userids[$event->userid])) {
-                $userids[$event->userid] = $event->userid;
-            }
-            if (!empty($logextra['realuserid']) && empty($userids[$logextra['realuserid']])) {
-                $userids[$logextra['realuserid']] = $logextra['realuserid'];
-            }
-            if (!empty($event->relateduserid) && empty($userids[$event->relateduserid])) {
-                $userids[$event->relateduserid] = $event->relateduserid;
-            }
-        }
-        $this->rawdata->close();
-
-        // Get user fullname and put that in return list.
-        if (!empty($userids)) {
-            list($usql, $uparams) = $DB->get_in_or_equal($userids);
-            $users = $DB->get_records_sql("SELECT id," . get_all_user_name_fields(true) . " FROM {user} WHERE id " . $usql,
-                    $uparams);
-            foreach ($users as $userid => $user) {
-                $this->userfullnames[$userid] = fullname($user);
-                unset($userids[$userid]);
-            }
-
-            // We fill the array with false values for the users that don't exist anymore
-            // in the database so we don't need to query the db again later.
-            foreach ($userids as $userid) {
-                $this->userfullnames[$userid] = false;
-            }
-        }
+        throw new coding_exception('update_users_and_courses_used() can not be used any more.');
     }
 }
